@@ -5,7 +5,6 @@ import datetime
 
 from dateutil.relativedelta import relativedelta
 from dateutil.rrule import YEARLY, MONTHLY, DAILY, HOURLY, MINUTELY, SECONDLY
-from django.http import HttpResponse
 from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
 from matplotlib.dates import AutoDateFormatter
 from matplotlib.dates import AutoDateLocator
@@ -15,6 +14,10 @@ from matplotlib.dates import rrulewrapper
 from matplotlib.figure import Figure
 from matplotlib.ticker import MaxNLocator
 
+from django.core.urlresolvers import reverse
+from django.http import HttpResponse
+import simplejson as json
+
 from lizard_map.matplotlib_settings import FONT_SIZE
 from lizard_map.matplotlib_settings import SCREEN_DPI
 
@@ -28,6 +31,49 @@ def _inches_from_pixels(pixels):
     return pixels / SCREEN_DPI
 
 
+def parse_identifier_json(identifier_json):
+    """Return dict of parsed identifier_json.
+
+    Converts keys to str.
+    TODO: .replace('%22', '"') in a better way
+    """
+
+    identifier_json = identifier_json.replace('%22', '"')
+    if not identifier_json:
+        return {}
+    result = {}
+    for k, v in json.loads(identifier_json).items():
+        result[str(k)] = v
+    return result
+
+
+def workspace_item_image_url(workspace_item_id, identifiers,
+                             strip_layout=False, session_graph_options=False):
+    """
+    Returns image url
+
+    Identifiers is a list of dicts
+    """
+    identifiers_copy = identifiers[:]
+    if strip_layout:
+        for identifier in identifiers_copy:
+            if 'layout' in identifier:
+                del identifier['layout']
+    identifier_json_list = [json.dumps(identifier).replace('"', '%22') for \
+                                identifier in identifiers_copy]
+    if session_graph_options:
+        img_url = reverse(
+            "lizard_map.workspace_item_image_session_graph_options",
+            kwargs={'workspace_item_id': workspace_item_id, })
+    else:
+        img_url = reverse(
+            "lizard_map.workspace_item_image",
+            kwargs={'workspace_item_id': workspace_item_id, })
+    img_url = img_url + '?' + '&'.join(['identifier=%s' % i for i in
+                                        identifier_json_list])
+    return img_url
+
+
 class LessTicksAutoDateLocator(AutoDateLocator):
     """Similar to matplotlib.date.AutoDateLocator, but with less ticks."""
 
@@ -146,8 +192,10 @@ class Graph(object):
         self.legend_width = 0.01  # no legend by default
         self.left_label_width = LEFT_LABEL_WIDTH / self.width
         self.bottom_axis_location = BOTTOM_LINE_HEIGHT / self.height
+        self.x_label_height = 0.0
+        self.legend_on_bottom_height = 0.0
         self.axes = self.figure.add_subplot(111)
-        self.fixup_axes()
+        self.axes.grid(True)
         # Date range
         # self.axes.set_xlim(date2num((self.start_date, self.end_date)))
 
@@ -160,6 +208,10 @@ class Graph(object):
                              x=self.left_label_width,
                              horizontalalignment='left')
 
+    def set_xlabel(self, xlabel):
+        self.axes.set_xlabel(xlabel)
+        self.x_label_height = BOTTOM_LINE_HEIGHT / self.height
+
     def fixup_axes(self):
         """Fix up the axes by limiting the amount of items."""
         available_width = self.width - LEFT_LABEL_WIDTH - LEGEND_WIDTH
@@ -171,7 +223,10 @@ class Graph(object):
         self.axes.xaxis.set_major_locator(locator)
         self.axes.xaxis.set_major_formatter(AutoDateFormatter(locator))
 
-        available_height = self.height - BOTTOM_LINE_HEIGHT
+        available_height = (self.height -
+                            BOTTOM_LINE_HEIGHT -
+                            self.x_label_height -
+                            self.legend_on_bottom_height)
         approximate_lines = int(available_height / (FONT_SIZE * 1.5))
         max_number_of_ticks = approximate_lines
         if max_number_of_ticks < 2:
@@ -179,23 +234,47 @@ class Graph(object):
         locator = MaxNLocator(nbins=max_number_of_ticks - 1)
         self.axes.yaxis.set_major_locator(locator)
 
+    def legend_space(self):
+        """reserve space for legend (on the right side). even when
+        there is no legend displayed"""
+        self.legend_width = LEGEND_WIDTH / self.width
+
     def legend(self, handles=None, labels=None):
         """
+        Displays legend. Default is right side, but if the width is
+        too small, it will display under the graph.
+
         handles is list of matplotlib objects (e.g. matplotlib.lines.Line2D)
         labels is list of strings
         """
-        self.legend_width = LEGEND_WIDTH / self.width
+        # experimental update: do not reserve space for legend by
+        # default, just place over graph. use legend_space to manually
+        # add space
+
         if handles is None and labels is None:
             handles, labels = self.axes.get_legend_handles_labels()
-        if labels:
+        if handles and labels:
+            # Determine 'small' or 'large'
+            if self.width < 500:
+                legend_loc = 4  # lower right
+                # approximation of legend height
+                self.legend_on_bottom_height = min(
+                    (len(labels) + 1) * BOTTOM_LINE_HEIGHT / self.height,
+                    0.5)
+
+            else:
+                legend_loc = 1  # Upper right'
+
             return self.figure.legend(
                 handles,
                 labels,
                 bbox_to_anchor=(1 - self.legend_width,
                                 0,  # self.bottom_axis_location
                                 self.legend_width,
+                                # 1 = Upper right of above bbox. Use 0 for
+                                # 'best'
                                 1),
-                loc=4,  # Lower right of above bbox.
+                loc=legend_loc,
                 fancybox=True,
                 shadow=True,
                )
@@ -203,10 +282,19 @@ class Graph(object):
          # TODO: get rid of the border around the legend.
 
     def http_png(self):
-        self.axes.set_position((self.left_label_width,
-                                self.bottom_axis_location,
-                                1 - self.legend_width - self.left_label_width,
-                                1 - 2 * self.bottom_axis_location))
+        """Output plot to png. Also calculates size of plot and put 'now'
+        line."""
+        self.fixup_axes()
+
+        axes_left = self.left_label_width
+        axes_bottom = (self.bottom_axis_location + self.x_label_height +
+                       self.legend_on_bottom_height)
+        axes_width = 1 - self.legend_width - self.left_label_width
+        axes_height = (1 - 2 * self.bottom_axis_location -
+                       self.x_label_height - self.legend_on_bottom_height)
+
+        self.axes.set_position((axes_left, axes_bottom,
+                                axes_width, axes_height))
 
         # Set date range
         # Somehow, the range cannot be set in __init__
