@@ -1,7 +1,67 @@
-// jslint configuration
+// jslint configuration; btw: don't put a space before 'jslint' below.
 /*jslint browser: true */
-/*global $, OpenLayers, window, updateLayer, stretchOneSidebarBox,
-reloadGraphs, fillSidebar, show_popup */
+/*global $, OpenLayers, window, updateLayer, updateLayers,
+stretchOneSidebarBox, reloadGraphs, fillSidebar, show_popup */
+
+var animationTimer;
+
+// if (typeof(console) === 'undefined') {
+//     // Prevents the firebug console from throwing errors in browsers other
+//     // than Firefox/Chrome/Chromium
+//     // From http://gist.github.com/384113
+//     var console = {};
+//     console.log = function () {};
+// }
+
+
+function setSliderDate(slider_value) {
+    $.ajax({
+        type: "POST",
+        url: $("#animation-slider").attr("data-ajax-path"),
+        data: "slider_value=" + slider_value,
+        success: function (data) {
+            // Update the date label span with the returned data
+            $('span#selected-date').html($.parseJSON(data));
+        }
+    });
+}
+
+
+function setUpAnimationSlider() {
+    var workspace_item_id;
+    $("#animation-slider").slider({
+        min: parseInt($("#animation-slider").attr("data-min"), 10),
+        max: parseInt($("#animation-slider").attr("data-max"), 10),
+        step: parseInt($("#animation-slider").attr("data-step"), 10),
+        value: parseInt($("#animation-slider").attr("data-value"), 10),
+        slide: function (event, ui) {
+            if (animationTimer) {
+                clearTimeout(animationTimer);
+            }
+            animationTimer = setTimeout(
+                function () {
+                    // Only run after nothing happened for 300ms.
+                    setSliderDate(ui.value);
+                },
+                300);
+        },
+        change: function (event, ui) {
+            setSliderDate(ui.value);
+            updateLayers();
+        }
+    });
+}
+
+
+function reloadMapActions() {
+    $(".map-actions").load(
+        "./ .map-action",
+        function () {
+            fillSidebar();
+            setUpAnimationSlider();
+        });
+}
+
 
 function setUpWorkspaceAcceptable() {
     // Set up draggability for current and future items.
@@ -37,8 +97,7 @@ function setUpWorkspaceAcceptable() {
             },
             function (workspace_id) {
                 updateLayer(workspace_id);
-                $(".map-actions").load("./ .map-action",
-                                       fillSidebar);
+                reloadMapActions();
             });
         stretchOneSidebarBox();
     });
@@ -104,11 +163,15 @@ function nothingFoundPopup() {
               2000);
 }
 
+function setUpGraphEditPopup() {
+    $(".graph_edit_trigger").overlay();
+}
+
 /*
 Empty the temp workspace
 */
 function setUpEmptyTempInteraction() {
-    $("span.workspace-empty-temp").live("click", function () {
+    $(".workspace-empty-temp").live("click", function () {
         var $workspace, url, workspace_item_id;
         $(this).css("cursor", "progress");
         $workspace = $(".workspace");
@@ -120,8 +183,7 @@ function setUpEmptyTempInteraction() {
             function (workspace_id) {
                 updateLayer(workspace_id);
                 // load new map actions
-                $(".map-actions").load("./ .map-action",
-                                       fillSidebar);
+                reloadMapActions();
                 // remove highlighting
                 $(".workspace-acceptable").removeClass("selected");
             }
@@ -154,6 +216,7 @@ function popup_click_handler(x, y, map) {
 }
 
 
+
 // Initialize all workspace actions.
 $(document).ready(function () {
     setUpWorkspaceAcceptable();
@@ -161,8 +224,11 @@ $(document).ready(function () {
     setUpDateChoice();
     setUpNotFoundPopup();
     setUpEmptyTempInteraction();
+    setUpAnimationSlider();
+    setUpGraphEditPopup();
+
     /* Workspace functions, requires jquery.workspace.js */
     $(".workspace").workspaceInteraction();
-    // $(".add-snippet").snippetInteraction(); // als het met live werkt kan het hier
+    $(".add-snippet").snippetInteraction(); // voor collage view, nu nog nutteloos voor popup
     // $("a.lizard-map-link").lizardMapLink();
 });
