@@ -3,7 +3,7 @@ import simplejson
 
 import mapnik
 import PIL.Image
-from django.core.urlresolvers import reverse
+import csv
 from django.db.models import Max
 from django.http import HttpResponse
 from django.template import RequestContext
@@ -13,12 +13,14 @@ from django.views.decorators.cache import never_cache
 import simplejson as json
 
 from lizard_map import coordinates
+from lizard_map.adapter import parse_identifier_json
 from lizard_map.daterange import current_start_end_dates
 from lizard_map.daterange import DateRangeForm
 from lizard_map.models import Workspace
 from lizard_map.models import WorkspaceItem
 from lizard_map.models import WorkspaceCollage
 from lizard_map.models import WorkspaceCollageSnippet
+from lizard_map.models import WorkspaceCollageSnippetGroup
 from lizard_map.workspace import WorkspaceManager
 
 # Workspace stuff
@@ -26,9 +28,12 @@ from lizard_map.workspace import WorkspaceManager
 
 def workspace(request,
               workspace_id,
-              javascript_click_handler=None,
+              javascript_click_handler='popup_click_handler',
               template='lizard_map/workspace.html'):
-    """Render page with one workspace"""
+    """Render page with one workspace.
+
+    workspaces in dictionary, because of ... ?
+    """
     workspace = get_object_or_404(Workspace, pk=workspace_id)
     date_range_form = DateRangeForm(
         current_start_end_dates(request, for_form=True))
@@ -117,6 +122,87 @@ def workspace_item_edit(request, workspace_item_id=None, visible=None):
     return HttpResponse(json.dumps(workspace_item.workspace.id))
 
 
+@never_cache
+def snippet_group_graph_edit(request, snippet_group_id):
+    """Edits snippet_group properties using post.
+    """
+    post = request.POST
+    title = post.get('title', None)
+    x_label = post.get('x_label', None)
+    y_label = post.get('y_label', None)
+    y_min = post.get('y_min', None)
+    y_max = post.get('y_max', None)
+    boundary_value = post.get('boundary_value', None)
+    aggregation_period = post.get('aggregation_period', None)
+
+    snippet_group = WorkspaceCollageSnippetGroup.objects.get(
+        pk=snippet_group_id)
+    if title is not None:
+        # Empty string is also good! it will force default title.
+        snippet_group.layout_title = title
+    if x_label is not None:
+        snippet_group.layout_x_label = x_label
+    if y_label is not None:
+        snippet_group.layout_y_label = y_label
+    if aggregation_period is not None:
+        snippet_group.aggregation_period = int(aggregation_period)
+    try:
+        snippet_group.layout_y_min = float(y_min)
+    except (ValueError, TypeError):
+        snippet_group.layout_y_min = None
+    try:
+        snippet_group.layout_y_max = float(y_max)
+    except (ValueError, TypeError):
+        snippet_group.layout_y_max = None
+    try:
+        snippet_group.boundary_value = float(boundary_value)
+    except (ValueError, TypeError):
+        snippet_group.boundary_value = None
+    snippet_group.save()
+    return HttpResponse('')
+
+
+def snippet_group_image(request, snippet_group_id, legend=True):
+    """Draws a single image for the snippet_group. The MUST be at
+    least 1 snippet in the group."""
+    snippet_group = WorkspaceCollageSnippetGroup.objects.get(
+        pk=snippet_group_id)
+    snippets = snippet_group.snippets.all()
+    identifiers = [snippet.identifier for snippet in snippets]
+
+    # Add aggregation_period to each identifier
+    for identifier in identifiers:
+        if not 'layout' in identifier:
+            identifier['layout'] = {}
+        identifier['layout']['aggregation_period'] = snippet_group.aggregation_period
+
+    # Add legend option ('layout' is always present).
+    if legend:
+        for identifier in identifiers:
+            identifier['layout']['legend'] = True
+
+    # Get width and height.
+    width = request.GET.get('width')
+    height = request.GET.get('height')
+    if width:
+        width = int(width)
+    else:
+        # We want None, not u''.
+        width = None
+    if height:
+        height = int(height)
+    else:
+        # We want None, not u''.
+        height = None
+
+    workspace_item = snippets[0].workspace_item
+    start_date, end_date = current_start_end_dates(request)
+    return workspace_item.adapter.image(identifiers,
+                                        start_date, end_date,
+                                        width, height,
+                                        layout_extra=snippet_group.layout())
+
+
 @never_cache
 def workspace_item_delete(request, object_id=None):
     """delete workspace item from workspace
@@ -158,7 +244,7 @@ def session_workspace_edit_item(request,
     return
 
 
-def popup_json(found, popup_id=None, collage=False, request=None):
+def popup_json(found, popup_id=None, hide_add_snippet=False, request=None):
     """Return html with info on list of 'found' objects.
 
     found: list of dictionaries {'distance': ..., 'timeserie': ...,
@@ -166,7 +252,6 @@ def popup_json(found, popup_id=None, collage=False, request=None):
 
     Note: identifier must be a dict. {'id': the_real_id}.
 
-    TODO: make better
     """
 
     result_html = ''
@@ -192,57 +277,35 @@ def popup_json(found, popup_id=None, collage=False, request=None):
         workspace_manager = WorkspaceManager(request)
         workspace_manager.load_workspaces()
         temp_workspaces = workspace_manager.workspaces['temp']
-        for temp_workspace in temp_workspaces:
-            for workspace_item in temp_workspace.workspace_items.filter(
-                visible=True):
-                temp_workspace_item_ids.append(workspace_item.id)
+        temp_workspace_items = WorkspaceItem.objects.filter(
+            workspace__in=temp_workspaces, visible=True)
+        temp_workspace_item_ids = [
+            workspace_item.id for workspace_item in temp_workspace_items]
+
     # Now display them.
     for workspace_item_id, display_group in display_groups.items():
-        identifier_json_list = []
-        header = ''
-        for display_object in display_group:
-            # timeserie = display_object['object']
-            name = display_object.get('name', 'Geen naam')
-            shortname = display_object.get('shortname', '')
-            workspace_item = display_object['workspace_item']
-            identifier_json = simplejson.dumps(display_object['identifier']).replace('"', '%22')
-            identifier_json_list.append(identifier_json)
+        # There MUST be at least one item in the group
+        workspace_item = display_group[0]['workspace_item']
+
+        # Check if this display object must have the option add_snippet
+        if hide_add_snippet or (workspace_item_id in temp_workspace_item_ids):
+            add_snippet = False
+        else:
+            add_snippet = True
+
+        # Add workspace_item name on top
+        # title = workspace_item.name
+
+        identifiers = [display_object['identifier'] for display_object
+                       in display_group]
+        # img_url = workspace_item_image_url(workspace_item.id, identifiers)
+
+        html_per_workspace_item = workspace_item.adapter.html(
+            identifiers=identifiers,
+            layout_options={'add_snippet': add_snippet,
+                            'legend': True},
+            )
 
-            # Add workspace_item on top
-            if not header:
-                header += '<div><strong>%s</strong></div>' % (
-                    workspace_item.name)
-            # Compose html header for each display object (experimental)
-            if collage or (workspace_item_id in temp_workspace_item_ids):
-                header += (
-                    '<div>%s</div>') % name
-            else:
-                header += (
-                    '<div>%s<a href="" ' +
-                    'class="add-snippet ss_chart_line_add ss_sprite" ' +
-                    'data-workspace-id="%d" data-workspace-item-id="%d" ' +
-                    'data-item-identifier=\'%s\' data-item-shortname="%s" ' +
-                    'data-item-name="%s">Voeg toe aan collage</a></div>') % (
-                    name,
-                    workspace_item.workspace.id,
-                    workspace_item.id,
-                    identifier_json,
-                    shortname,
-                    name,
-                    )
-        #if not timeserie.data_count():
-        #    body = "<div>Geen gegevens beschikbaar.</div>"
-        #else:
-        img_url = reverse("lizard_map.workspace_item_image",
-                      kwargs={'workspace_item_id': workspace_item.id,
-                              })
-        img_url = img_url + '?' + '&'.join(['identifier=%s' % i for i in
-                                            identifier_json_list])
-        body = ('<div style="width: 100%%; height: 250px;" ' +
-                # Yeah, that's a double %: template escaping.
-                'class="img-use-my-size">' +
-                '<a href="%s" class="replace-with-image" /></div>') % img_url
-        html_per_workspace_item = header + body
         x_found, y_found = display_object['google_coords']
         result_html += html_per_workspace_item
 
@@ -252,7 +315,41 @@ def popup_json(found, popup_id=None, collage=False, request=None):
               'x': x_found,
               'y': y_found,
               'html': result_html,
-              'big': big_popup
+              'big': big_popup,
+              }
+    return HttpResponse(simplejson.dumps(result))
+
+
+def popup_collage_json(collage, popup_id, request=None):
+    """
+    display collage by adding display_groups together
+    TODO: must work with items from same adapter, not only workspace_item
+    """
+
+    result_html = ''
+    snippet_groups = collage.snippet_groups.all()
+    if len(snippet_groups) > 1:
+        big_popup = True
+    else:
+        big_popup = False
+
+    for snippet_group in snippet_groups:
+        snippets = snippet_group.snippets.all()
+        if snippets:
+            # Pick workspace_item of first snippet to build html
+            workspace_item = snippets[0].workspace_item
+            html_per_workspace_item = workspace_item.adapter.html(
+                snippet_group=snippet_group,
+                layout_options={'legend': True})
+            result_html += html_per_workspace_item
+            # Pick the location of the first snippet.
+            google_x, google_y = snippets[0].location['google_coords']
+
+    result = {'id': popup_id,
+              'x': google_x,
+              'y': google_y,
+              'html': result_html,
+              'big': big_popup,
               }
     return HttpResponse(simplejson.dumps(result))
 
@@ -261,12 +358,18 @@ def popup_json(found, popup_id=None, collage=False, request=None):
 
 
 def collage(request,
-              collage_id,
-              template='lizard_map/collage.html'):
+            collage_id,
+            editable=False,
+            template='lizard_map/collage.html'):
     """Render page with one collage"""
+    date_range_form = DateRangeForm(
+        current_start_end_dates(request, for_form=True))
     return render_to_response(
         template,
-        {'collage_id': collage_id},
+        {'collage': get_object_or_404(WorkspaceCollage, pk=collage_id),
+         'editable': editable,
+         'date_range_form': date_range_form,
+         'request': request},
         context_instance=RequestContext(request))
 
 
@@ -293,17 +396,20 @@ def session_collage_snippet_add(request,
             'workspace_item_location_name')
 
     workspace_id = request.session['workspaces'][workspace_category][0]
-    workspace = get_object_or_404(Workspace, pk=workspace_id)
-    workspace_item = get_object_or_404(WorkspaceItem, pk=workspace_item_id)
+    workspace = Workspace.objects.get(pk=workspace_id)
+    workspace_item = WorkspaceItem.objects.get(pk=workspace_item_id)
 
     if len(workspace.collages.all()) == 0:
         workspace.collages.create()
     collage = workspace.collages.all()[0]
-    collage.snippets.get_or_create(
+
+    # create snippet using collage function: also finds/makes snippet group
+    snippet, _ = collage.get_or_create_snippet(
         workspace_item=workspace_item,
         identifier_json=workspace_item_location_identifier,
         shortname=workspace_item_location_shortname,
         name=workspace_item_location_name)
+
     return HttpResponse(json.dumps(workspace_id))
 
 
@@ -316,8 +422,9 @@ def session_collage_snippet_delete(request,
     if object_id is None:
         object_id = request.POST.get('object_id')
     snippet = get_object_or_404(WorkspaceCollageSnippet, pk=object_id)
-    snippet.delete()
+    # snippet_group = snippet.snippet_group
 
+    snippet.delete()
     return HttpResponse()
 
 
@@ -332,7 +439,8 @@ def snippet_popup(request, snippet_id=None):
     popup_id = 'popup-snippet-%s' % snippet_id
     return popup_json([snippet.location, ],
                       popup_id=popup_id,
-                      request=request)
+                      request=request,
+                      hide_add_snippet=True)
 
 
 def collage_popup(request,
@@ -343,11 +451,12 @@ def collage_popup(request,
         collage_id = request.GET.get('collage_id')
     collage = get_object_or_404(WorkspaceCollage, pk=collage_id)
     popup_id = 'popup-collage'
+
     # Only one collage popup allowed, also check jquery.workspace.js
-    return popup_json(collage.locations,
-                      popup_id=popup_id,
-                      collage=True,
-                      request=request)
+    return popup_collage_json(
+        collage,
+        popup_id=popup_id,
+        request=request)
 
 
 def workspace_item_image(request, workspace_item_id):
@@ -370,15 +479,46 @@ def workspace_item_image(request, workspace_item_id):
     else:
         # We want None, not u''.
         height = None
-    print "%r, %r" % (width, height)
+
     workspace_item = get_object_or_404(WorkspaceItem, pk=workspace_item_id)
     start_date, end_date = current_start_end_dates(request)
-    return workspace_item.adapter.image(identifier_list, start_date, end_date, width, height)
+    return workspace_item.adapter.image(identifier_list, start_date, end_date,
+                                        width, height)
+
+
+def snippet_edit(request, snippet_id):
+    """
+    Edits snippet layout properties.
+    """
+    snippet = WorkspaceCollageSnippet.objects.get(pk=snippet_id)
+
+    post = request.POST
+    layout = {}
+    if post.get('color', None):
+        layout.update({'color': post['color']})
+    if post.__contains__('line_min'):
+        layout.update({'line_min': None})
+    if post.__contains__('line_max'):
+        layout.update({'line_max': None})
+    if post.__contains__('line_avg'):
+        layout.update({'line_avg': None})
+
+    identifier = snippet.identifier
+    if 'layout' in identifier:
+        del identifier['layout']
+    identifier.update({'layout': layout})
+    # print identifier
+    snippet.set_identifier(identifier)
+    snippet.save()
+
+    return HttpResponse('')
+
 
 """
 Map stuff
 """
 
+
 def wms(request, workspace_id):
     """Return PNG as WMS service."""
 
@@ -400,7 +540,8 @@ def wms(request, workspace_id):
     #m.background = mapnik.Color('blue')
 
     for workspace_item in workspace.workspace_items.filter(visible=True):
-        layers, styles = workspace_item.adapter.layer(layer_ids=layers)
+        layers, styles = workspace_item.adapter.layer(layer_ids=layers,
+                                                      request=request)
         layers.reverse()  # first item should be drawn on top (=last)
         for layer in layers:
             mapnik_map.layers.append(layer)
@@ -445,7 +586,6 @@ def search_coordinates(request):
                 search_results = workspace_item.adapter.search(
                     google_x, google_y, radius=google_radius)
                 found += search_results
-
     if found:
         # ``found`` is a list of dicts {'distance': ..., 'timeserie': ...}.
         found.sort(key=lambda item: item['distance'])
@@ -476,3 +616,54 @@ def clickinfo(request, workspace_id):
         msg = 'Nothing found'
     # TODO: return json: {msg, x_found, y_found}
     return HttpResponse(msg)
+
+
+"""
+Export
+"""
+
+
+def export_identifier_csv(request, workspace_item_id=None,
+    identifier_json=None):
+    """
+    Uses adapter.values to get values. Then return these values in csv format.
+    """
+    if workspace_item_id is None:
+        workspace_item_id = request.GET.get('workspace_item_id')
+    if identifier_json is None:
+        identifier_json = request.GET.get('identifier_json')
+    workspace_item = WorkspaceItem.objects.get(pk=workspace_item_id)
+    identifier = parse_identifier_json(identifier_json)
+    start_date, end_date = current_start_end_dates(request)
+    values = workspace_item.adapter.values(identifier, start_date, end_date)
+
+    response = HttpResponse(mimetype='text/csv')
+    response['Content-Disposition'] = 'attachment; filename=export.csv'
+    writer = csv.writer(response)
+    for row in values:
+        writer.writerow([row['datetime'], row['value'], row['unit']])
+    return response
+
+
+def export_snippet_group_statistics_csv(request, snippet_group_id=None):
+    """
+    Exports snippet_group statistics as csv.
+    """
+    if snippet_group_id is None:
+        snippet_group_id = request.GET.get('snippet_group_id')
+    snippet_group = WorkspaceCollageSnippetGroup.objects.get(
+        pk=snippet_group_id)
+    start_date, end_date = current_start_end_dates(request)
+    statistics = snippet_group.statistics(start_date, end_date)
+
+    response = HttpResponse(mimetype='text/csv')
+    response['Content-Disposition'] = 'attachment; filename=export.csv'
+    writer = csv.writer(response)
+    colnames = ['min', 'max', 'avg', 'count_lt', 'count_gte',
+                'percentile', 'percentile_75']
+    colnamesdisplay = ['min', 'max', 'avg', '< boundary', '>= boundary',
+                       'percentile_25', 'percentile_75']
+    writer.writerow(colnamesdisplay)
+    for row in statistics:
+        writer.writerow([row[colname] for colname in colnames])
+    return response
